// Mock the imported modules
jest.mock('./modules/collapsible.js', () => ({
    Collapsible: jest.fn()
  }));

  jest.mock('./modules/removeitem.js', () => ({
    removeItem: jest.fn(),
  }));

  
  jest.mock('./modules/helpers.js', () => ({
    copyToClipboard: jest.fn(),
    createItem: jest.fn(),
    toggleCollapsibleMetas: jest.fn(),
    handleInputChange: jest.fn(),
  }));
  
  jest.mock('./modules/reveal.js', () => ({
    initializeRevealTargets: jest.fn(),
    setupRevealConditions: jest.fn(),
    handleRevealConditions: jest.fn()
  }));

  // Import the mocked modules
  import { Collapsible } from './modules/collapsible.js';
  import { copyToClipboard, createItem, toggleCollapsibleMetas, handleInputChange } from './modules/helpers.js';
  import { removeItem } from './modules/removeitem.js';
  import { initializeRevealTargets, setupRevealConditions, handleRevealConditions } from './modules/reveal.js';

  describe('modules.js', () => {
    beforeEach(() => {
      // Clear all mocks before each test
      jest.clearAllMocks();

      // Reset the window object
      global.window = {};

      // Run the code in modules.js
      require('./modules.js');
    });

  
    test('Collapsible is initialized with correct selectors', () => {
      expect(Collapsible).toHaveBeenCalledWith({
        collapsibleWrapper: '.collapsible-wrapper',
        clickableSelector: '.toggle-help-section__cta',
        contentSelector: '.help-section',
        textSelector: '.toggle-help-section__cta'
      });
    });

    test('RemoveItem functions are correctly assigned to window object', () => {
      expect(window.removeItem).toBe(removeItem);
    });

    test('Helper functions are correctly assigned to window object', () => {
      expect(window.copyToClipboard).toBe(copyToClipboard);
      expect(window.createItem).toBe(createItem);
      expect(window.toggleCollapsibleMetas).toBe(toggleCollapsibleMetas);
      expect(window.handleInputChange).toBe(handleInputChange);
    });

    test('Reveal functions are correctly assigned to window object', () => {
      expect(window.initializeRevealTargets).toBe(initializeRevealTargets);
      expect(window.setupRevealConditions).toBe(setupRevealConditions);
      expect(window.handleRevealConditions).toBe(handleRevealConditions);
    });
  });