function CollapsibleElement({ collapsible, options }) {
  const $ctaEl = collapsible.querySelector(options?.clickableSelector || ".collapse-cta");
  const $contentEl = collapsible.querySelector(options?.contentSelector || ".collapse-content");
  const $arrow = collapsible.querySelector(".collapsible-icon");
  const $initialText = collapsible.querySelector(`${options?.textSelector}-text.initial`);
  const $secondaryText = collapsible.querySelector(`${options?.textSelector}-text.secondary`);
  const shouldSwapTexts = !!($initialText && $secondaryText);

    if (!$ctaEl || !$contentEl) return;

  // Improved visibility check for content element
  const isContentVisible = () =>
    window.getComputedStyle($contentEl).display !== "none";

  const toggleCollapsible = () => {
    if (isContentVisible()) {
      $contentEl.style.display = "none";
      arrowHandler(false, $arrow);
      textHandler(shouldSwapTexts, false);
    } else {
      $contentEl.style.display = "block";
      arrowHandler(true, $arrow);
      textHandler(shouldSwapTexts, true);
    }
  };

  const textHandler = (shouldSwap, visible) => {
    if (!shouldSwap) return;
    if (visible) {
      $initialText.style.display = "inline";
      $secondaryText.style.display = "none";
    } else {
      $initialText.style.display = "none";
      $secondaryText.style.display = "inline";
    }
  };

  const arrowHandler = (visible, arrow) => {
    console.log(visible, arrow);
    if (!arrow) return;
    visible
      ? arrow.classList.replace("tykon-arrowdown", "tykon-arrowup")
      : arrow.classList.replace("tykon-arrowup", "tykon-arrowdown");
  };

  $ctaEl.onclick = toggleCollapsible;
}

export function Collapsible(options) {
  const collapsibleElements = document.querySelectorAll(
    options?.collapsibleWrapper || ".default-collapsible-class"
  );

  const init = () => {
    collapsibleElements.forEach(collapsible => { CollapsibleElement({ collapsible, options }); });
  };

  init();
}
