export function copyToClipboard(elemId) {
    const element = document.getElementById(elemId);
    let textToCopy;
  
    if (element.tagName.toLowerCase() === 'input') {
      textToCopy = element.value;
    } else if (element.tagName.toLowerCase() === 'div') {
      textToCopy = element.innerText;
    }
  
    navigator.clipboard.writeText(textToCopy)
      .catch(err => {
        console.error('Failed to copy text: ', err);
      });
}

export function toggleCollapsibleMetas(elem) {
  const targetedElms = document.querySelectorAll(elem.dataset.collapsibleTarget);
  targetedElms.forEach(el => {
    el.style.display = el.style.display === "none" ? "block" : "none";
  });
  elem.classList.toggle('tykon-arrowdown');
  elem.classList.toggle('tykon-arrowup');
}

export function handleInputChange(event) {
  const input = event.target;
  const isDynamicSetter = input.dataset.isDynamicSetter === 'true';
  const targetClass = input.dataset.targetClass;
  const placeholder = input.dataset.placeholder;

  input?.classList?.remove('error-highlight');

  if (!isDynamicSetter) return;

  let elementsToUpdate = document.querySelectorAll(targetClass);
  let formattedValue = input.value.toLowerCase().replace(/ /g, "-");

  elementsToUpdate.forEach(element => {
    if (element.tagName === 'INPUT' || element.tagName === 'TEXTAREA') {
      element.value = element.value.replace(new RegExp(placeholder, 'g'), formattedValue);
    } else if (element.tagName === 'DIV') {
      element.innerText = element.innerText.replace(new RegExp(placeholder, 'g'), formattedValue);
    } else {
      element.textContent = element.textContent.replace(new RegExp(placeholder, 'g'), formattedValue);
    }
  });

  input.dataset.placeholder = formattedValue;
}

export function createItem(value, {selector, selectorWrapper, valueSelector}, inputElm) {
  let wrapper = inputElm.closest(selectorWrapper);
  
  if (!wrapper) {
    console.error('Wrapper not found:', selectorWrapper);
    return;
  }
  
  let input = wrapper.parentElement.querySelector(valueSelector);
  
  if (!input) {
    console.error('Hidden input not found:', valueSelector);
    return;
  }
  
  if (!input.value.split(",").includes(value)) {
    let select = wrapper.querySelector(selector);
    
    if (!select) {
      console.error('Select element not found:', selector);
      return;
    }
    
    let newItem = document.createRange().createContextualFragment(`<div class="select2-selection__choice" data-input="${valueSelector}"><span class="select2-selection__choice__remove" onclick="removeItem(this)" role="presentation">×</span>${value}</div>`);
    wrapper.insertBefore(newItem, select);
    
    if (input.value == "") {
      input.value = value;
    } else {
      input.value += `,${value}`;
    }
    
  } 
}