import { initializeRevealTargets } from './reveal';

// Mock DOM methods
document.querySelectorAll = jest.fn();

describe('initializeRevealTargets', () => {
  beforeEach(() => {
    jest.clearAllMocks();
  });

  test('should hide all elements with data-reveal-target attribute', () => {
    const mockElements = [
      { style: { display: 'block' } },
      { style: { display: 'inline' } },
      { style: { display: 'flex' } }
    ];
    document.querySelectorAll.mockReturnValue(mockElements);

    initializeRevealTargets();

    expect(document.querySelectorAll).toHaveBeenCalledWith('[data-reveal-target]');
    mockElements.forEach(el => {
      expect(el.style.display).toBe('none');
    });
  });

  test('should do nothing when no elements are found', () => {
    document.querySelectorAll.mockReturnValue([]);

    initializeRevealTargets();

    expect(document.querySelectorAll).toHaveBeenCalledWith('[data-reveal-target]');
  });

  test('should handle multiple elements correctly', () => {
    const mockElements = [
      { style: { display: 'block' } },
      { style: { display: 'inline' } },
      { style: { display: 'flex' } },
      { style: { display: 'grid' } },
      { style: { display: 'inline-block' } }
    ];
    document.querySelectorAll.mockReturnValue(mockElements);

    initializeRevealTargets();

    expect(document.querySelectorAll).toHaveBeenCalledWith('[data-reveal-target]');
    mockElements.forEach(el => {
      expect(el.style.display).toBe('none');
    });
  });
});

