export function initializeRevealTargets() {
    document.querySelectorAll('[data-reveal-target]').forEach(el => {
        el.style.display = 'none';
    });
}


export function setupRevealConditions(select) {
    const conditions = JSON.parse(select.dataset.revealConditions);
    const fieldset = select.closest('.qor-fieldset');
    
    const update = () => {
      const matchingCondition = conditions.find(condition => condition.Value === select.value);
      
      if (matchingCondition) {
        showMatchingTarget(matchingCondition, fieldset);
      }
    };
  
    update();
  
    return update;
}

export function handleRevealConditions(selectElement) {
  let conditions;
  try {
    conditions = JSON.parse(selectElement.getAttribute('data-reveal-conditions'));
  } catch (error) {
    console.error('Error parsing JSON:', error);
    return;
  }
  
  const currentValue = selectElement.value;
  const fieldset = selectElement.closest('.qor-fieldset');
  
  // Hide all potential targets
  hideAllTargets(conditions, fieldset);
  
  // Show matching target
  const matchingCondition = conditions.find(condition => condition.Value === currentValue);
  if (matchingCondition) {
    showMatchingTarget(matchingCondition, fieldset);
  }
}

function hideAllTargets(conditions, fieldset) {
  conditions.forEach(({Target}) => {
    const targetElements = fieldset.querySelectorAll(`.${Target}`);
    targetElements.forEach(targetElement => {
      hideElement(targetElement);
      handleNestedSelects(targetElement, fieldset);
    });
  });
}

function hideElement(element) {
  element.style.display = 'none';
  element.setAttribute('data-reveal-target', '');
}

function handleNestedSelects(element, fieldset) {
  const nestedSelects = element.querySelectorAll('select[data-reveal-conditions]');
  nestedSelects.forEach(nestedSelect => {
    hideNestedTargets(nestedSelect, fieldset);
  });
}

function hideNestedTargets(nestedSelect, fieldset) {
  try {
    const nestedConditions = JSON.parse(nestedSelect.getAttribute('data-reveal-conditions'));
    nestedConditions.forEach(({Target}) => {
      const nestedElements = fieldset.querySelectorAll(`.${Target}`);
      nestedElements.forEach(hideElement);
    });
  } catch (error) {
    console.error('Error parsing nested conditions:', error);
  }
}

function showMatchingTarget(matchingCondition, fieldset) {
  const targetElements = fieldset.querySelectorAll(`.${matchingCondition.Target}`);
  targetElements.forEach(targetElement => {
    showElement(targetElement);
    processNestedSelects(targetElement);
  });
}

function showElement(element) {
  element.style.display = '';
  element.removeAttribute('data-reveal-target');
}

function processNestedSelects(element) {
  const nestedSelects = element.querySelectorAll('select[data-reveal-conditions]');
  nestedSelects.forEach(handleRevealConditions);
}