import { toggleCollapsibleMetas } from './helpers';

// Mock DOM methods
document.querySelectorAll = jest.fn();

// Mock classList
const mockClassList = {
  toggle: jest.fn()
};

describe('toggleCollapsibleMetas', () => {
  beforeEach(() => {
    jest.clearAllMocks();
  });

  test('should toggle display from none to block', () => {
    const mockElement = { style: { display: 'none' } };
    document.querySelectorAll.mockReturnValue([mockElement]);

    const mockElem = {
      dataset: { collapsibleTarget: '.test-target' },
      classList: mockClassList
    };

    toggleCollapsibleMetas(mockElem);

    expect(document.querySelectorAll).toHaveBeenCalledWith('.test-target');
    expect(mockElement.style.display).toBe('block');
    expect(mockClassList.toggle).toHaveBeenCalledWith('tykon-arrowdown');
    expect(mockClassList.toggle).toHaveBeenCalledWith('tykon-arrowup');
  });

  test('should toggle display from block to none', () => {
    const mockElement = { style: { display: 'block' } };
    document.querySelectorAll.mockReturnValue([mockElement]);

    const mockElem = {
      dataset: { collapsibleTarget: '.test-target' },
      classList: mockClassList
    };

    toggleCollapsibleMetas(mockElem);

    expect(document.querySelectorAll).toHaveBeenCalledWith('.test-target');
    expect(mockElement.style.display).toBe('none');
    expect(mockClassList.toggle).toHaveBeenCalledWith('tykon-arrowdown');
    expect(mockClassList.toggle).toHaveBeenCalledWith('tykon-arrowup');
  });

  test('should toggle display for multiple targeted elements', () => {
    const mockElements = [
      { style: { display: 'none' } },
      { style: { display: 'block' } },
      { style: { display: 'none' } }
    ];
    document.querySelectorAll.mockReturnValue(mockElements);

    const mockElem = {
      dataset: { collapsibleTarget: '.test-target' },
      classList: mockClassList
    };

    toggleCollapsibleMetas(mockElem);

    expect(document.querySelectorAll).toHaveBeenCalledWith('.test-target');
    expect(mockElements[0].style.display).toBe('block');
    expect(mockElements[1].style.display).toBe('none');
    expect(mockElements[2].style.display).toBe('block');
    expect(mockClassList.toggle).toHaveBeenCalledWith('tykon-arrowdown');
    expect(mockClassList.toggle).toHaveBeenCalledWith('tykon-arrowup');
  });
});