$(function() {
    let _ = window._,
        QOR = window.QOR,
        QOR_Translations = window.QOR_Translations,
        $dialog, 
        html = `<div id="dialog" style="display: none;">
                  <div class="mdl-dialog-bg"></div>
                  <div class="mdl-dialog">
                      <div class="mdl-dialog__content">
                        <p><i class="tyk-icon tykon tykon-warning"></i></p>
                        <p class="mdl-dialog__message dialog-message">
                        </p>
                      </div>
                      <div class="mdl-dialog__actions">
                        <button type="button" class="tyk-button tyk-button--success tyk-button--md dialog-ok dialog-button" data-type="confirm">
                          ${QOR_Translations.okButton}
                        </button>
                        <button type="button" class="tyk-button tyk-button--primary-outline tyk-button--md dialog-cancel dialog-button" data-type="">
                          ${QOR_Translations.cancelButton}
                        </button>
                      </div>
                    </div>
                </div>`,
        deleteHtml =  `<div id="dialog" style="display: none;">
                        <div class="mdl-dialog-bg"></div>
                        <div class="mdl-dialog mdl-dialog--delete">
                            <div class="mdl-dialog__content mdl-dialog__content--delete">
                                <div class="mdl-dialog__title"><i class="tyk-hexagon-exclamation"></i></div>
                                <div class="mdl-dialog__message dialog-message"></div>
                            </div>
                            <span class="mdl-dialog-divider"></span>
                            <div class="mdl-dialog__actions mdl-dialog__actions--delete">
                                <button type="button" class="tyk-button portal-critical tyk-button--md dialog-ok dialog-button" data-type="confirm">
                                    ${QOR_Translations.okButton}
                                </button>
                                <button type="button" class="tyk-button tyk-button--secondary-outline tyk-button--md dialog-cancel dialog-button" data-type="">
                                    ${QOR_Translations.cancelButton}
                                </button>
                            </div>
                        </div>
                    </div>`;

    // ************************************ Refactor window.confirm ************************************
    $(document)
        .on("keyup.qor.confirm", function(e) {
            if (!$dialog || !$dialog.is(":visible")) {
                return;
            }
            if (e.which === 27) {
                setTimeout(function() {
                    $dialog.remove();
                    QOR.qorConfirmCallback = undefined;
                }, 100);
            }
            if (e.which === 13) {
                setTimeout(function() {
                    $('.dialog-button[data-type="confirm"]').click();
                }, 100);
            }
        })
        .on("click.qor.confirm", ".dialog-button", function() {
            let value = $(this).data("type"),
                callback = QOR.qorConfirmCallback;

            $.isFunction(callback) && callback(value);
            $dialog.remove();
            QOR.qorConfirmCallback = undefined;
            return false;
        });

    let sanitizeHTML = function(html) {
        html = html.replace(/<script\b[^<]*(?:(?!<\/script>)<[^<]*)*<\/script>/gi, '')
                   .replace(/<iframe\b[^<]*(?:(?!<\/iframe>)<[^<]*)*<\/iframe>/gi, '')
                   .replace(/<object\b[^<]*(?:(?!<\/object>)<[^<]*)*<\/object>/gi, '')
                   .replace(/<embed\b[^<]*(?:(?!<\/embed>)<[^<]*)*<\/embed>/gi, '')
                   .replace(/<style\b[^<]*(?:(?!<\/style>)<[^<]*)*<\/style>/gi, '')
                   .replace(/on\w+="[^"]*"/gi, '')
                   .replace(/javascript:[^\s>]*/gi, '');

        return html;
    };

    QOR.qorConfirm = function(data, callback) {
        $("#dialog").remove();
        if (data.customModal === "delete") {
            $dialog = $(deleteHtml).appendTo("body");
        } else {
            $dialog = $(html).appendTo("body");
        }

        let okBtn = $dialog.find(".dialog-ok"),
            cancelBtn = $dialog.find(".dialog-cancel");

        if (_.isString(data)) {
            $dialog.find(".dialog-message").text(data);
            okBtn.text(QOR_Translations.okButton);
            cancelBtn.text(QOR_Translations.cancelButton);
        } else if (_.isObject(data)) {
            if (data.confirmOk && data.confirmCancel) {
                okBtn.text(data.confirmOk);
                cancelBtn.text(data.confirmCancel);
            } else {
                okBtn.text(QOR_Translations.okButton);
                cancelBtn.text(QOR_Translations.cancelButton);
            }

            if (data.icon) {
                $dialog.find('i.material-icons').addClass(data.icon).html(data.icon);
            }

            $dialog.find(".dialog-message").html(sanitizeHTML(data.confirm));
        }

        $dialog.show();
        QOR.qorConfirmCallback = callback;
        return false;
    };

    // *******************************************************************************

    // ****************Handle download file from AJAX POST****************************
    let objectToFormData = function(obj, form) {
        let formdata = form || new FormData(),
            key;

        for (var variable in obj) {
            // if (obj.hasOwnProperty(variable) && obj[variable]) {
            if (Object.prototype.hasOwnProperty.call(obj, variable) && obj[variable]) {
                key = variable;
            }

            if (obj[variable] instanceof Date) {
                formdata.append(key, obj[variable].toISOString());
            } else if (
                typeof obj[variable] === "object" &&
                !(obj[variable] instanceof File)
            ) {
                objectToFormData(obj[variable], formdata);
            } else {
                formdata.append(key, obj[variable]);
            }
        }

        return formdata;
    };

    QOR.qorAjaxHandleFile = function(url, contentType, fileName, data) {
        let request = new XMLHttpRequest();

        request.responseType = "arraybuffer";
        request.open("POST", url, true);
        request.onload = function() {
            if (this.status === 200) {
                let blob = new Blob([this.response], {
                        type: contentType
                    }),
                    url = window.URL.createObjectURL(blob),
                    a = document.createElement("a");

                document.body.appendChild(a);
                a.href = url;
                a.download = fileName || "download-" + $.now();
                a.click();
            } else {
                window.alert(QOR_Translations.serverError);
            }
        };

        if (_.isObject(data)) {
            if (Object.prototype.toString.call(data) != "[object FormData]") {
                data = objectToFormData(data);
            }

            request.send(data);
        }
    };

    // ********************************convert video link********************
    // linkyoutube: /https?:\/\/(?:[0-9A-Z-]+\.)?(?:youtu\.be\/|youtube\.com\S*[^\w\-\s])([\w\-]{11})(?=[^\w\-]|$)(?![?=&+%\w.\-]*(?:['"][^<>]*>|<\/a>))[?=&+%\w.-]*/ig,
    // linkvimeo: /https?:\/\/(www\.)?vimeo.com\/(\d+)($|\/)/,

    let converVideoLinks = function() {
        let $ele = $(".qor-linkify-object"),
            linkyoutube = /https?:\/\/(?:[0-9A-Z-]+\.)?(?:youtu\.be\/|youtube\.com\S*[^\w\-\s])([\w\-]{11})(?=[^\w\-]|$)(?![?=&+%\w.\-]*(?:['"][^<>]*>|<\/a>))[?=&+%\w.-]*/gi;

        if (!$ele.length) {
            return;
        }

        $ele.each(function() {
            let url = $(this).data("video-link");
            if (url.match(linkyoutube)) {
                $(this).html(
                    `<iframe width="100%" height="100%" src="//www.youtube.com/embed/${url.replace(
            linkyoutube,
            "$1"
          )}" frameborder="0" allowfullscreen></iframe>`
                );
            }
        });
    };

    $.fn.qorSliderAfterShow.converVideoLinks = converVideoLinks;
    converVideoLinks();

    // ********************************Qor Handle AJAX error********************
    QOR.handleAjaxError = function(err) {
        let isSlideoutOpen = !!$(".qor-slideout.is-shown.is-slided").length > 0;
        let $body = isSlideoutOpen ? $(".qor-slideout__body") : $(".qor-page__body"),
            rJSON = err.responseJSON,
            rText = err.responseText,
            $error = $(`<ul class="qor-alert qor-error qor-alert__active" data-dismissible="true"><button type="button" class="mdl-button mdl-button--icon" data-dismiss="alert">
                            <i class="tyk-icon tykon tykon-x"></i>
                        </button></ul>`);

        $body.find(".qor-alert:not(.qor-alert-action-button)").remove();

        if (rJSON) {
            let errors = rJSON.errors,
                $errorContent = "";

            if (Array.isArray(errors)) {
                for (let i = 0; i < errors.length; i++) {
                    $errorContent += `<li>
                                      <i class="material-icons">error</i>
                                      <span>${errors[i]}</span>
                                  </li>`;
                }
            } else {
                $errorContent = `<li>
                          <i class="material-icons">error</i>
                          <span>${errors}</span>
                      </li>`;
            }
            $error.append($errorContent);
        } else {
            $error = $(rText).find(".qor-error:not(.qor-error-action-button)");
        }

        $error.prependTo($body);
    };

    // ********************************Display alert popup on error to retry********************
    QOR.handleAjaxAlert = function(msg, handler, url) {
        let isSlideoutOpen = !!$(".qor-slideout.is-shown.is-slided").length > 0;
        let $body = isSlideoutOpen ? $(".qor-slideout__body") : $(".mdl-layout__container"),
            $error = $(`<div id="dialog" style="display: block;">
            <div class="mdl-dialog-bg"></div>
            <div class="mdl-dialog">
                <div class="mdl-dialog__content">
                  <p><i class="tyk-icon tykon tykon-warning"></i></p>
                  <p class="mdl-dialog__message dialog-message">${msg}</p>
                </div>
                <div class="mdl-dialog__actions">
                  <button type="button" class="tyk-button tyk-button--success tyk-button--md dialog-ok dialog-button" data-type="confirm">Yes</button>
                  <button type="button" class="tyk-button tyk-button--primary-outline tyk-button--md dialog-cancel dialog-button" data-type="">No</button>
                </div>
              </div>
          </div>`);

        $body.find(".qor-alert").remove();

        $error.prependTo($body);
        $error.find(".dialog-cancel").on("click", function() {
            $error.remove();
        });
        $error.find(".dialog-ok").on("click", function() {
            handler(url);
            $error.remove();
        });
    };
});