(function(factory) {
    if (typeof define === "function" && define.amd) {
        // AMD. Register as anonymous module.
        define(["jquery"], factory);
    } else if (typeof exports === "object") {
        // Node / CommonJS
        factory(require("jquery"));
    } else {
        // Browser globals.
        factory(jQuery);
    }
})(function($) {
    "use strict";

    let NAMESPACE = "qor.markdown",
        EVENT_ENABLE = "enable." + NAMESPACE,
        EVENT_DISABLE = "disable." + NAMESPACE;

    function QorMarkdown(element, options) {
        this.$element = $(element);
        this.options = $.extend({}, QorMarkdown.DEFAULTS, $.isPlainObject(options) && options);
        this.init();
    }

    QorMarkdown.prototype = {
        constructor: QorMarkdown,

        init: function() {
            let $this = this.$element;
            let id = $this.attr('id');
            let element = $this[0];
            this.ed = new window.EasyMDE({
                element: element,
                // TODO: make this an option
                autosave: {
                    enabled: true,
                    delay: 1000,
                    uniqueId: id,
                },
            });
            this.ed.value(element.innerText);
            let easymde = this.ed;
            let targetElement = element.parentNode;
            while (targetElement !== null) {
                if (targetElement.classList && targetElement.classList.contains("collapse-content")) {
                    break;
                }
                targetElement = targetElement.parentNode;
            }
            const observer = new MutationObserver((mutationsList) => {
                for (const mutation of mutationsList) {
                    if (mutation.target === targetElement && mutation.attributeName === 'style') {
                        const displayStyle = window.getComputedStyle(targetElement) && window.getComputedStyle(targetElement).getPropertyValue('display');
                        if(displayStyle === "block") {
                            easymde.codemirror.refresh();
                        }
                    }
                }
            });
            if (targetElement) observer.observe(targetElement, { attributes: true});
            this.bind();
        },

        bind: function() {},

        unbind: function() {}
    };

    QorMarkdown.plugin = function(option) {
        return this.each(function() {
            let $this = $(this);
            let data = $this.data(NAMESPACE);
            let fn;

            if (!data) {
                if (!window.EasyMDE) {
                    return;
                }

                if (/destroy/.test(option)) {
                    return;
                }

                $this.data(NAMESPACE, (data = new QorMarkdown($this, option)));
            }

            if (typeof option === 'string' && $.isFunction((fn = data[option]))) {
                fn.apply(data);
            }
        });
    };

    QorMarkdown.DEFAULTS = {};

    $(function() {
        var selector = 'textarea[data-toggle="qor.markdown"]';
        $(document)
            .on(EVENT_DISABLE, function(e) {
                QorMarkdown.plugin.call($(selector, e.target), "destroy");
            })
            .on(EVENT_ENABLE, function(e) {
                let element = $(selector, e.target);
                QorMarkdown.plugin.call(element);
            })
            .triggerHandler(EVENT_ENABLE);
    });

    return QorMarkdown;
});