const CLASS_OPEN = 'qor-modal-open',
  ARIA_HIDDEN = 'aria-hidden';
/* On form cancel */
function onCancel() {
  let paths = window.location.pathname.split('/').filter((path) => path);
  let backURL = window.location.origin + '/' + paths[0] + '/' + paths[1];

  /* If it has opened on a new page, needs to go back with page reload otherwise without */
  if ($('.qor-slideout').hasClass('is-slided')) {
    window.history.pushState({}, null, backURL);
  } else {
    window.location.href = backURL;
  }
}

function CodeReferenceOnChange(inputId, value, crType) {
  const codeRefContainer = document.getElementById(`${inputId}_container`);
  codeRefContainer.style.display = 'flex';
  const codeRefEl = document.getElementById(inputId);
  let crValue;
  switch (crType) {
    case 'content':
      crValue = `safe .blocks.${value}.Content`;
      break;
    case 'image':
      crValue = `.blocks.${value}.Image.URL`;
      break;
    default:
      crValue = `.blocks.${value}.Content`;
      break;
  }
  codeRefEl.value = value == '' ? '' : `{{ ${crValue} }}`;
}

function hideHelpSection() {
  let overviewToggleButton = document.querySelector('.toggle-help-section');
  let overviewToggleSection = document.querySelector('.help-section');

  overviewToggleButton.style.display = 'none';
  overviewToggleSection.style.display = 'none';
  localStorage.setItem('hideHelpSection', 'hidden');
}

document.addEventListener(
  'DOMContentLoaded',
  function () {
    let overviewToggleButton = document.querySelector('.toggle-help-section');
    let overviewToggleSection = document.querySelector('.help-section');

    if (localStorage.getItem('hideHelpSection') === 'hidden' && overviewToggleButton) {
      overviewToggleButton.style.display = 'none';
      overviewToggleSection.style.display = 'none';
    }
  },
  false
);

function revealValue(elemId, emelValueId) {
  const codeRefEl = document.getElementById(elemId);
  const codeRefElValue = document.getElementById(emelValueId);
  if (codeRefEl.value === codeRefElValue.value) codeRefEl.value = '******';
  else codeRefEl.value = codeRefElValue.value;
}

function actionButtonToggleModal(id) {
  var modal = document.getElementById(id);
  if (modal.classList.contains('hidden')) {
    modal.classList.remove('hidden');
    modal.classList.add('shown');
    $(document).find('body').addClass(CLASS_OPEN);
    $(modal).attr(ARIA_HIDDEN, false);
  } else {
    modal.classList.add('hidden');
    modal.classList.remove('shown');
    $(document).find('body').removeClass(CLASS_OPEN);
    $(modal).attr(ARIA_HIDDEN, true);
  }
}

function actionButtonToggleResult(id, response) {
      if (response.status == 200) {
          let successResult = document.getElementById(id + "-success");

          if (successResult != null) {
            successResult.classList.add("qor-alert__active");
          }
      } else {
          let errorResult = document.getElementById(id + "-error");
          let errorSpan = errorResult?.querySelector('span');
          if (errorSpan && response.status) {
            let statusText = response.statusText || '';
            if (errorSpan.textContent.includes('Response status code:')) {
              errorSpan.textContent = errorSpan.textContent.replace(/Response status code:.*/, `Response status code: ${response.status} ${statusText}`);
            } else {
             errorSpan.textContent = `${errorSpan.textContent} Response status code: ${response.status} ${statusText}`;
            }
          }
          errorResult?.classList.add("qor-alert__active");
      }
}

function actionButtonExecute(reloadPage, actionbuttonModalId, url, method, data, datatype) {
  $.ajax(url, {
    xhrFields: {
      withCredentials: true,
    },
    method,
    data,
    dataType: datatype || 'json',
    beforeSend: function (xhr) {
      let successResult = document.getElementById(actionbuttonModalId + "-success");

      if (successResult != null) {
          successResult.classList.remove("qor-alert__active");
      }

      let errorResult = document.getElementById(actionbuttonModalId + "-error");

      if (errorResult != null) {
          errorResult.classList.remove("qor-alert__active");
      }

      console.log('beforeSend');
      console.log(data)
    },
    success: function (data) {
      console.log('success.data:', data);
    },
    error: function (err) {
      if (err.status == 200) {
        return;
      }
    },
    complete: function (response) {
      actionButtonToggleResult(actionbuttonModalId, response);

      if (reloadPage) {
        window.location.reload();
      }
    },
  });
}

function actionButtonExecuteParseForm(reloadPage, actionbuttonModalId, url, method, data, datatype) {
  actionButtonExecute(reloadPage, actionbuttonModalId, url, method, actionButtonParseForm(data), datatype);
}


function actionButtonParseForm(params) {
  let data = [];

  for (let i = 0; i < params.length; i++) {
    let e = document.getElementsByName(`QorResource.${params[i].value}`);

    if (e != null && e.length != 0) {
      let d = {};

      d["name"] = params[i].name;

      if (params[i].type == "checkbox") {
        d["value"] = e[0].checked.toString();
      } else {
        d["value"] = e[0].value;
      }

      data.push(d)
    }
  }

  return data;
}

function actionHandler(name, menu_id, id, move) {
  let path = name.split(",")
  let url = `/admin/${path[0]}/${menu_id}/${path[1]}/${id}/${move}`
  actionButtonExecute(false,null,url,"PUT")
}

(function(factory) {
    if (typeof define === 'function' && define.amd) {
        // AMD. Register as anonymous module.
        define(['jquery'], factory);
    } else if (typeof exports === 'object') {
        // Node / CommonJS
        factory(require('jquery'));
    } else {
        // Browser globals.
        factory(jQuery);
    }
})(function($) {
    'use strict';

    let NAMESPACE = 'qor.collection.sortable',
        EVENT_ENABLE = 'enable.' + NAMESPACE,
        EVENT_DISABLE = 'disable.' + NAMESPACE,
        EVENT_CLICK = 'click.' + NAMESPACE,
        CLASS_ITEM = '.qor-sortable__item',
        CLASS_CHILDREN_ITEM = '> .qor-field__block > .qor-sortable__item',
        CLASS_BUTTON_CHANGE = '>.qor-sortable__button > .qor-sortable__button-change',
        CLASS_BUTTON_DONE = '>.qor-sortable__button > .qor-sortable__button-done',
        CLASS_BUTTON_ADD = '> .qor-field__block > .qor-sortable__button-add',
        CLASS_BUTTON_DELETE = '> .qor-sortable__button-delete',
        CLASS_BUTTON_MOVE = '>.qor-field__block > .qor-sortable__item > .qor-sortable__change .qor-sortable__button-move',
        CLASS_ACTION = '> .qor-sortable__change > .qor-sortable__action',
        CLASS_ACTION_POSITION = '.qor-sortable__action-position',
        CLASS_ITEM_FILTER = '.is-delete,.qor-fieldset--new',
        IS_LOADED = 'sortable-collection-loaded';

    function moveArrPos(arr, from, name) {
      switch(name){
        case "move_up":
          if (from > 0) {
            let toVal = arr[from-1]
            let fromVal = arr[from]
            arr[from] = toVal
            arr[from-1] = fromVal
          }
          break
        case "move_down":
          if (from < arr.length) {
            let toVal = arr[from+1]
            let fromVal = arr[from]
            arr[from] = toVal
            arr[from+1] = fromVal
          }
          break
      }
      return arr;
    }

    function QorCollectionSortable(element, options) {
        this.$element = $(element);
        this.options = $.extend({}, QorCollectionSortable.DEFAULTS, $.isPlainObject(options) && options);
        this.init();
    }

    QorCollectionSortable.prototype = {
        constructor: QorCollectionSortable,

        init: function() {
            this.bind();
            this.initItemOrder();
        },

        bind: function() {
            this.$element
                .on(EVENT_CLICK, CLASS_BUTTON_MOVE, this.moveItem.bind(this))
                .on(EVENT_CLICK, CLASS_BUTTON_DONE, this.finishMoveItem.bind(this))
                .on(EVENT_CLICK, CLASS_BUTTON_CHANGE, this.startMoveItem.bind(this));

            $(document).on('added.qor.replicator.sortableCollection', this.resetOrderArr.bind(this));
        },

        unbind: function() {
            this.$element
                .off(EVENT_CLICK, CLASS_BUTTON_MOVE)
                .off(EVENT_CLICK, CLASS_BUTTON_DONE)
                .off(EVENT_CLICK, CLASS_BUTTON_CHANGE);
            $(document).off('added.qor.replicator.sortableCollection');
        },

        initItemOrder: function(resetResource) {
            let $item = this.$element.find(CLASS_CHILDREN_ITEM).not(CLASS_ITEM_FILTER);

            // return false if no item
            if (!$item.length) {
                return;
            }

            this.$item = $item;

            let $select = $item.find(CLASS_ACTION).find(CLASS_ACTION_POSITION),
                orderData = {},
                orderArr = [],
                itemTotal = $item.length,
                template = $item.first().html(),
                fullResourceName,
                resourceNamePrefix;

            if ($select.length) {
                $select.remove();
            }

            if (!resourceNamePrefix) {
                fullResourceName = template.match(/(\w+)\="(\S*\[\d+\]\S*)"/); // get results : [attribute, name, value]
                if (fullResourceName.length) {
                    fullResourceName = fullResourceName[2];
                    resourceNamePrefix = fullResourceName.match(/^(\S*)\[(\d+)\]([^\[\]]*)$/); // get results : [input, prefix, index, suffix]

                    if (resourceNamePrefix.length) {
                        resourceNamePrefix = resourceNamePrefix[1];
                    }
                }
            }

            $item.each(function() {
                let $this = $(this),
                    $action = $this.find(CLASS_ACTION),
                    itemIndex = parseInt($this.attr('order-index'));

                orderData.itemTotal = itemTotal;
                orderData.itemIndex = itemIndex;
                orderArr.push(itemIndex);

                /* $action.prepend('<select class="qor-sortable__action-position"></select>'); */

                for (let i = 1; i <= itemTotal; i++) {
                    let renderData = {},
                        isSelected;

                    itemIndex + 1 == i ? (isSelected = true) : (isSelected = false);
                    renderData = $.extend({}, orderData, {selectorPosition: i, isSelected: isSelected});

                    $action.find('select').append(window.Mustache.render(QorCollectionSortable.OPTION_HTML, renderData));
                }

                // reset form resource name prop
                if (resetResource) {
                    let resourceName,
                        newPosition,
                        resourceNameEnd,
                        hasPrefixPosition = /\[\d+\]/.test(resourceNamePrefix),
                        $resource = $(this).find('[name^="' + resourceNamePrefix + '"]');

                    if (!$resource.length) {
                        return;
                    }

                    $resource.each(function() {
                        resourceName = $(this).prop('name');
                        resourceNameEnd = resourceName.match(/\.\w+$/);
                        newPosition = `[${itemIndex}]`;

                        if (hasPrefixPosition) {
                            // fullResourceName = QorResource.SerializableMeta.Sections[0].Items[0(this number is the position)].Name
                            // resourceNamePrefix = QorResource.SerializableMeta.Sections[0].Items
                            // change the last [0].Name to real position
                            resourceName = resourceName.replace(/\[\d+\]\.\w+$/, `${newPosition}${resourceNameEnd}`);
                        } else {
                            resourceName = resourceName.replace(/\[\d+\]/, newPosition);
                        }

                        $(this).prop('name', resourceName);
                    });
                }

                $this.data(orderData);
            });
            this.itemOrderArr = this.itemOrderArr || orderArr;
        },

        resetOrderArr: function(e, $item) {
            let orderIndex = parseInt($item.attr('order-index'));
            if (!orderIndex) {
                return;
            }

            this.itemOrderArr = this.itemOrderArr || [];
            this.itemOrderArr.push(orderIndex);
            this.$item = this.$element.find(CLASS_CHILDREN_ITEM).not(CLASS_ITEM_FILTER);
        },

        moveItem: function(e) {
            let $current = $(e.target).closest(CLASS_ITEM),
                currentPosition = $current.data().itemIndex,
                targetPosition = parseInt($current.find(CLASS_ACTION_POSITION).val()) - 1,
                $item = this.$item,
                name = $(e.target).attr('name'),
                newPosArr = [];
            /* if (targetPosition == currentPosition) {
                return;
            } */

            newPosArr = moveArrPos(this.itemOrderArr, currentPosition, name);

            newPosArr.forEach(function(arr) {
                let index = newPosArr.indexOf(arr);

                $item
                    .filter(`[order-item="item_${arr}"]`)
                    .attr('order-index', index)
                    .css('order', index);
            });
            this.itemOrderArr = newPosArr;
            this.initItemOrder(true);
        },

        finishMoveItem: function(e) {
            let $target = $(e.target),
                $element = this.$element,
                $item = $element.find(CLASS_CHILDREN_ITEM).not(CLASS_ITEM_FILTER),
                $addButton = $element.find(CLASS_BUTTON_ADD),
                $deleteButton = $item.find(CLASS_BUTTON_DELETE),
                $actionButtons = $item.find(CLASS_ACTION);

            $target.hide();
            $actionButtons.hide();

            $element.find(CLASS_BUTTON_CHANGE).show();
            $addButton.show();
            $deleteButton.show();
        },

        startMoveItem: function(e) {
            let $target = $(e.target),
                $element = this.$element,
                $item = $element.find(CLASS_CHILDREN_ITEM).not(CLASS_ITEM_FILTER),
                $addButton = $element.find(CLASS_BUTTON_ADD),
                $deleteButton = $item.find(CLASS_BUTTON_DELETE),
                $actionButtons = $item.find(CLASS_ACTION);

            if (!$item.length) {
                return false;
            }

            $target.hide();

            $element.find(CLASS_BUTTON_DONE).show();
            $actionButtons.show();
            $addButton.hide();
            $deleteButton.hide();
            this.initItemOrder();
        },

        destroy: function() {
            this.unbind();
            this.$element.removeData(NAMESPACE);
        }
    };

    QorCollectionSortable.DEFAULTS = {};

    QorCollectionSortable.OPTION_HTML = '<option value="[[selectorPosition]]" [[#isSelected]]selected[[/isSelected]]>[[selectorPosition]] of [[itemTotal]]</option>';

    QorCollectionSortable.plugin = function(options) {
        return this.each(function() {
            var $this = $(this);
            var data = $this.data(NAMESPACE);
            var fn;

            if (!data) {
                if (/destroy/.test(options)) {
                    return;
                }

                $this.data(NAMESPACE, (data = new QorCollectionSortable(this, options)));
            }

            if (typeof options === 'string' && $.isFunction((fn = data[options]))) {
                fn.apply(data);
            }
        });
    };

    $(function() {
        var selector = '[data-toggle="qor.collection.sortable"]';

        if ($('body').data(IS_LOADED)) {
            return;
        }
        $('body').data(IS_LOADED, true);

        $(document)
            .on(EVENT_DISABLE, function(e) {
                QorCollectionSortable.plugin.call($(selector, e.target), 'destroy');
            })
            .on(EVENT_ENABLE, function(e) {
                QorCollectionSortable.plugin.call($(selector, e.target));
            })
            .triggerHandler(EVENT_ENABLE);
    });

    return QorCollectionSortable;
});

function toggleRedactor(showClass, hideClass) {
  $(showClass).show();
  $(hideClass).hide();
  if (showClass == ".markdow-editor-wrapper") {
    let htmlCol = document.querySelectorAll(showClass);
    htmlCol.forEach(e => {
      let editors = e.getElementsByClassName('EasyMDEContainer');
      if (editors.length >=2) {
        editors[editors.length-1].parentNode.removeChild(editors[editors.length-1])
      }
    });
  }
}


function waitForElm(selector, selectAll) {
  return new Promise(resolve => {
    if (document.querySelector(selector)) {
      let res = selectAll ? document.querySelectorAll(selector) : document.querySelector(selector)
      return resolve(res);
    }

    const observer = new MutationObserver(mutations => {
        if (document.querySelector(selector)) {
          let res = selectAll ? document.querySelectorAll(selector) : document.querySelector(selector)
          resolve(res);
          observer.disconnect();
        }
    });

    observer.observe(document.body, {
      childList: true,
      subtree: true
    });
  });
};

function removeAllChildren(element, className) {
  let children = element.querySelectorAll('.' + className);
  children.forEach(function(child) {
    child.parentNode.removeChild(child);
  });
}

function autocomplete(inp, arr, createNew, selector, selectorWrapper, valueSelector) {
  let currentFocus;
  let parent = inp.parentNode;
  let isListOpen = false;
  let crypto = window.crypto || window.msCrypto;
  let autocompleteListId = `autocomplete-list-${inp.id || crypto.getRandomValues(new Uint8Array(1))[0].toString(16).padStart(2, '0')}`;
  closeAllLists();

  function createAutocompleteList() {
    closeAllLists();
    isListOpen = true;
    var autocompleteList = document.createElement("DIV");
    autocompleteList.setAttribute("id", autocompleteListId);
    autocompleteList.setAttribute("class", "autocomplete-items");
    autocompleteList.style = "width:100%";
    parent.appendChild(autocompleteList);
    return autocompleteList;
  }

  parent.addEventListener("click", function(e) {
    if (isListOpen) {
      isListOpen = false;
      closeAllLists();
    } else {
      autocompleteList = createAutocompleteList();
      for (var i = 0; i < arr.length; i++) {
        var option = document.createElement("DIV");
        option.innerHTML = arr[i].Name;
        option.innerHTML += `<input type="hidden" value="${arr[i].Name}">`
        option.style.cursor = "pointer";
        option.classList.add("autocomplete-tag-item");
        option.addEventListener("click", function(e) {
          let value = this.getElementsByTagName("input")[0].value;
          inp.value = "";
          createItem(value, {selector, selectorWrapper, valueSelector}, inp);
          closeAllLists();
        });
        autocompleteList.appendChild(option);
      }
    }
  });

  inp.addEventListener("input", function(e) {
    var val = this.value;
    closeAllLists();
    if (!val) {
      return false;
    }
    currentFocus = -1;
    autocompleteList = createAutocompleteList();
    for (var i = 0; i < arr.length; i++) {
      if (arr[i].Name.substr(0, val.length).toLowerCase() == val.toLowerCase()) {
        var option = document.createElement("DIV");
        option.innerHTML = "<strong>" + arr[i].Name.substr(0, val.length) + "</strong>";
        option.innerHTML += arr[i].Name.substr(val.length);
        option.innerHTML += "<input type='hidden' value='" + arr[i].Name + "'>";
        option.style.cursor = "pointer";
        option.classList.add("autocomplete-tag-item");
        option.addEventListener("click", function(e) {
          let value = this.getElementsByTagName("input")[0].value;
          inp.value = "";
          createItem(value, {selector, selectorWrapper, valueSelector}, inp);
          closeAllLists();
        });
        autocompleteList.appendChild(option);
      }
    }
  });

  inp.addEventListener("keydown", function(e) {
    var x = document.getElementById(autocompleteListId);
    if (x) x = x.getElementsByTagName("div");
    if (e.keyCode == 40) {
      currentFocus++;
      addActive(x);
    } else if (e.keyCode == 38) { 
      currentFocus--;
      addActive(x);
    } else if (e.keyCode == 13) {
      e.preventDefault();
      if (currentFocus > -1) {
        if (x) x[currentFocus].click();
      }
      if (createNew) {
        let value = e.target.value.trim();
        if (!inp.dataset.caseSensitive) {
          value = value.toLowerCase();
        }
        if (value !== "") {
          createItem(value, {selector, selectorWrapper, valueSelector}, inp);
          inp.value = "";
        }
      }
    }
  });

  function addActive(x) {
    if (!x) return false;
    removeActive(x);
    if (currentFocus >= x.length) currentFocus = 0;
    if (currentFocus < 0) currentFocus = (x.length - 1);
    x[currentFocus].classList.add("autocomplete-active");
    x[currentFocus].classList.remove("autocomplete-tag-item");
  }

  function removeActive(x) {
    for (var i = 0; i < x.length; i++) {
      x[i].classList.remove("autocomplete-active");
      x[i].classList.add("autocomplete-tag-item");
    }
  }

  function closeAllLists(elmnt) {
    var x = document.getElementsByClassName("autocomplete-items");
    for (var i = 0; i < x.length; i++) {
      if (elmnt != x[i] && elmnt != inp) {
        x[i].parentNode.removeChild(x[i]);
      }
    }
  }

  document.addEventListener("click", function (e) {
    closeAllLists(e.target);
  });
}

function dropDownValuesInput(inp, selector, selectorWrapper, valueSelector) {
  inp.addEventListener("keydown", function(e) {
  if (e.keyCode == 13) {
      e.preventDefault();
      let value = e.target.value.trim();
      if (value !== "") {
        let inputElm = inp.parentNode.parentNode.querySelector(valueSelector);
        createItem(value, {selector, selectorWrapper, valueSelector, inputElm});
        inp.value = "";
      }
    }
  });
}
function InitDinamicWatcher(selectorWrapper, options) {
  waitForElm(selectorWrapper, false).then(wrapper => {
    const observer = new MutationObserver((mutationsList) => {
      for (const mutation of mutationsList) {
        if (mutation.type === 'attributes' && mutation.attributeName === 'class') {
          const classList = wrapper.classList;
          if (
            classList.contains('is-shown') &&
            classList.contains('is-slided')
          ) {
            for (const option of options) {
              waitForElm(option.selector, option.selectAll).then(element => option.observeCallback(element))
            }
          }
        }
      }
    });
  
    observer.observe(wrapper, { attributes: true });
  })
}

function observeAutocompleteDinamicSelect(elm) {
  let data = elm.dataset.dinamic.split("|");
  let dinamicData = elm.dataset.dinamic.split(".");
  let resPath = dinamicData[dinamicData.length - 1].toLowerCase();
  let dinamicElem = document.querySelector(`select[name="${data[1]}"]`);
  let initialMutation = true;
  const observer = new MutationObserver((mutationsList) => {
    for (const mutation of mutationsList) {
      if (mutation.type === 'attributes' && mutation.attributeName === 'chooser-selected') {
        if (!initialMutation) {
          let selectMany = document.querySelector(".dinamic-selection--multiple");
          removeAllChildren(selectMany, "select2-selection__choice");
          let input = document.querySelector('.dinamic-select-input-value');
          input.value = "";
        }
        let resID = resPath.substring(0, 3) === "org" ? `${resPath.substring(0, 3)}:${mutation.target.value}` : `${mutation.target.value}`;
        let url = `/admin/${resPath}s/${resID}.json`;
        $.ajax(url, {
          method: 'get',
          dataType: 'json',
          success: function (res) {
            autocomplete(elm, res[data[0]], false, ".dinamic-select-many", ".dinamic-selection--multiple", ".dinamic-select-input-value");
          },
          error: function (xhr, textStatus, errorThrown) {
            autocomplete(elm, false);
            if (xhr.status === 422) {
              console.log(xhr.responseText);
            } else {
              console.log([textStatus, errorThrown].join(': '));
            }
          }
        });
        initialMutation = false;
      }
    }
  });

  observer.observe(dinamicElem, { attributes: true });
}

function observeAutocompleteTag(elm) {
  let tagsURL = elm.dataset.tagsUrl;
  if (tagsURL) {
    $.ajax(tagsURL, {
      method: 'get',
      dataType: 'json',
      success: function(res) {
        autocomplete(elm,res, true, ".tags-select-many", ".tags-selection--multiple", ".tags-input-value")
      },
      error: function(xhr, textStatus, errorThrown) {
          if (xhr.status === 422) {
              window.alert(xhr.responseText);
          } else {
              window.alert([textStatus, errorThrown].join(': '));
          }
          location.reload();
      }
    })
  }else{
    autocomplete(elm, [], true, ".tags-select-many", ".tags-selection--multiple", ".tags-input-value")
  }
  
}

function displayComponent(elm, selectorClosest, display) {
  let parent = elm.closest(selectorClosest);
  for (let componentSelector of display.show) {
    let component = parent.querySelector(componentSelector);
    $(component).show();
  }
  for (let componentSelector of display.hide) {
    let component = parent.querySelector(componentSelector);
    $(component).hide();
  }
}

function handleDropDownValues(elm, selectorClosest) {
  let parent = elm.closest(selectorClosest);
  let input = parent.querySelector(".tags-select-many");
  dropDownValuesInput(input, ".tags-select-many", ".tags-selection--multiple", ".tags-input-value");
}


function observeCustomAttributeTypeEdit(elms) {
  const selectorClosest = ".collapsible-wrapper";
  const dropDownValues = ".dropdown-values-component";
  const attrRegexp = ".custom-attribute-regexp";
  const enableValidation = ".custom-attribute-regexp-enable";

  elms.forEach( elm => {
    if (elm.innerText == "Dropdown") {
      displayComponent(elm, selectorClosest, {show: [dropDownValues], hide: []});
      handleDropDownValues(elm, selectorClosest);
    }else if (elm.innerText == "String" || elm.innerText == "Number") {
      displayComponent(elm, selectorClosest, {show: [attrRegexp, enableValidation], hide: []});
    }
  });
}

function observeCustomAttributeTypeNew(elm) {
  const observer = new MutationObserver(function (mutationsList, observer) {
    for (const mutation of mutationsList) {
      if (mutation.type === 'childList' && mutation.addedNodes.length > 0) {
        const addedNodesArray = Array.from(mutation.addedNodes);
        for (const addedNode of addedNodesArray) {
          if (addedNode.nodeType === Node.ELEMENT_NODE && addedNode.classList.contains('qor-fieldset')) {
            let dropdownElm = addedNode.querySelector(".custom-attribute-type > span > span.selection > span > span.select2-selection__rendered");
            dropDownOnChangeTextHandler(dropdownElm);
          }
        }
      }
    }
  });
  observer.observe(elm, {childList: true, subtree: true})
}

function dropDownOnChangeTextHandler(targetElement) {
  const observer = new MutationObserver(function (mutationsList, observer) {
    for (const mutation of mutationsList) {
      if (mutation.type === 'childList' && mutation.target === targetElement) {
        const selectorClosest = ".collapsible-wrapper";
        const dropDownValues = ".dropdown-values-component";
        const attrRegexp = ".custom-attribute-regexp";
        const enableValidation = ".custom-attribute-regexp-enable";
        if (targetElement.innerText == "Dropdown") {
          displayComponent(targetElement, selectorClosest, {show: [dropDownValues], hide: [attrRegexp, enableValidation]});
          handleDropDownValues(targetElement, selectorClosest);
        } else if (targetElement.innerText == "String" || targetElement.innerText == "Number") {
          displayComponent(targetElement, selectorClosest, {show: [attrRegexp, enableValidation], hide: [dropDownValues]});
        } else if (targetElement.innerText == "Boolean") {
          displayComponent(targetElement, selectorClosest, {show: [], hide: [attrRegexp, enableValidation,dropDownValues]});
        }
      }
    }
  });
  observer.observe(targetElement, { childList: true, subtree: true });
}

InitDinamicWatcher(".qor-slideout", [{selector: ".dinamic-select-many", selectAll: false, observeCallback:  observeAutocompleteDinamicSelect}, {selector: ".custom-attribute-type > span > span.selection > span", selectAll: true, observeCallback: observeCustomAttributeTypeEdit}, {selector: ".qor-field__block", selectAll: false, observeCallback: observeCustomAttributeTypeNew}]);

function showTooltip(cell) {
  const tooltip = cell.querySelector('.row-tooltip');
  const arrow = cell.querySelector('.top-bubble-arrow');
  tooltip.style.display = 'block';
  arrow.style.display = 'block';
}

function hideTooltip(cell) {
  const tooltip = cell.querySelector('.row-tooltip');
  const arrow = cell.querySelector('.top-bubble-arrow');
  tooltip.style.display = 'none';
  arrow.style.display = 'none';
}

const toggleCollapsible = (elem) => {
  const collapsible = elem.parentNode;
  const ctaEl = collapsible.querySelector(".collapse-cta");
  const contentEl = collapsible.querySelector(".collapse-content");
  const arrow = collapsible.querySelector(".collapsible-icon");
  const isContentVisible = () => window.getComputedStyle(contentEl).display !== "none";
  if (isContentVisible()) {
    contentEl.style.display = "none";
    arrowHandler(false, arrow);
  } else {
    contentEl.style.display = "block";
    arrowHandler(true, arrow);
  }
};

const arrowHandler = (visible, arrow) => {
  if (!arrow) return;
  visible
    ? arrow.classList.replace("tykon-arrowdown", "tykon-arrowup")
    : arrow.classList.replace("tykon-arrowup", "tykon-arrowdown");
};


document.addEventListener('DOMContentLoaded', () => {
  // Check if we're in a QOR tabs view
  const isQorTabsView = document.querySelector('[data-toggle="qor.tabs"]');
  
  // If it's a QOR tabs view, return early and let qor-tabs.js handle the reveal logic
  if (isQorTabsView) {
    return;
  }

  // Otherwise, proceed with the standard reveal logic
  initializeRevealTargets();

  const updateFunctions = [];
  document.querySelectorAll('select[data-reveal-conditions]').forEach(select => {
    updateFunctions.push(setupRevealConditions(select));
  });

  const form = document.querySelector('form.qor-form');
  if (form) {
    form.addEventListener('change', (event) => {
      if (event.target.matches('select[data-reveal-conditions]')) {
        updateFunctions.forEach(update => update());
      }
    });
  }  
});



